*! version 1.1.1 PR 21mar2006.
program define mfpboot
version 8
gettoken cmd 0 : 0
frac_chk `cmd' 
if `s(bad)' {
	di as err "invalid or unrecognised command, `cmd'"
	exit 198
}
/*
	dist=0 (normal), 1 (binomial), 2 (poisson), 3 (cox), 4 (glm),
	5 (xtgee), 6(ereg/weibull), 7(streg,stcox).
*/
local dist `s(dist)'
local glm `s(isglm)'
local qreg `s(isqreg)'
local xtgee `s(isxtgee)'
local normal `s(isnorm)'

global MFpdist `dist'

* Disentangle
GetVL `0'
local 0 `s(nought)'

local yvar $MFP_dv
* construct RHS for mfp. `rhs' is actual variables in use,
* `RHS' may include brackets for joint selection in mfp.
local rhs $MFP_cur
local RHS
forvalues i=1/$MFP_n {
	local v ${MFP_`i'}
	if wordcount("`v'")>1 {
		local RHS `RHS' (`v')
	}
	else local RHS `RHS' `v'
}

* CLEAR option is needed because mfpboot may overwrite covariate(s) without warning.
syntax [if] [in] [aw fw iw pw], CLEAR OUTfile(string) [ REPlicates(int 100) /*
 */ ADJust(str) BAsedata DEAD(varname) DF(str) DFDefault(int 4) REPLACE noRESample /*
 */ noSCAling SEEd(int -1) SAVing(string) KEEPalso(varlist) * ]
if (`replicates'<1 & "`basedata'"=="") | (`replicates'<0 & "`basedata'"!="") {
	di as err "invalid replicates()"
	exit 198
}
if `dist'==7 & `"`saving'"'!="" {
	di as err "saving() not supported for st survival data"
	exit 198
}
if "`keepalso'"!="" & `"`saving'"'=="" {
	di as err "keepalso() invalid without saving()"
	exit 198
}
if `seed'>-1 {
	set seed `seed'
}
tempvar touse
quietly {
	local nx: word count `rhs'
	marksample touse
	markout `touse' `varlist' `dead'
	if "`dead'"!="" {
		local options "`options' dead(`dead')"
	}
/*
	Adjustment---must be frozen here to avoid different
	adjustments in each replicate. Also store names of predictors.
*/
	frac_adj "`adjust'" "`rhs'" `touse'
	local adjust
	local i 1
	while `i'<=`nx' {
		local uniq`i'=r(uniq`i')
		local adj`i' `r(adj`i')'
		local i=`i'+1
	}
	local i 1
	while `i'<=`nx' {
		local v`i': word `i' of `rhs'
		local adj `adj`i''
		if "`adj'"!="no" & "`adj'"!="" {
			if "`adj'"=="mean" {
				sum `v`i'' if `touse', meanonly
				local adj=r(mean)
			}
			else {
				noi confirm num `adj'
			}
		}
		local adj`i' `adj'
		local i=`i'+1
	}
/*
	Degrees of freedom for each variable
*/
	if "`df'"!="" {
		frac_dis "`df'" df 1 . "`rhs'"
		local i 1
		while `i'<=`nx' {
			if "${S_`i'}"!="" {
				local df`i' ${S_`i'}
			}
			local i=`i'+1
		}
	}
/*
	Assign default df for vars not so far accounted for.
	Give 1 df if 2-3 distinct values, 2 df for 4-5 values,
	dfdefault df for >=6 values.
*/
	local pb
	local failed 0
	local i 1
	while `i'<=`nx' {
		if "`df`i''"=="" {
			if `uniq`i''<=3 {
				local df`i' 1
			}
			else if `uniq`i''<=5 {
				local df`i'=min(2,`dfdefault')
			}
			else local df`i' `dfdefault'
		}
		local np`i'=cond(`df`i''==1,1,int(.1+`df`i''/2))
		local xs`i' `v`i'' /* was =substr("`v`i''",1,12), abbreviation abandoned */
		local j 1
		while `j'<=`np`i'' {
			local pb `pb' `xs`i''p`j' `xs`i''b`j' 
			local j=`j'+1
		}
		local i=`i'+1
	}
	if "`df'"!="" {
		local df df(`df')
	}
	local options `options' dfdefault(`dfdefault') `df'
/*
	Shift/scale/adjust: determine here,
	apply to all boot replications and store
*/
	local i 1
	while `i'<=`nx' {
		local sh`i' 0			/* shift */
		local sc`i' 1			/* scale */
		local V `v`i''
		if `df`i''>1 {
			fracgen `V' 0 if `touse', nogen `scaling'
			if r(shift)!=0 | r(scale)!=1 {
				local sh`i'=r(shift)
				local sc`i'=r(scale)
				local lab: var lab `v`i''
				lab var `v`i'' "`lab' WARNING: CHANGED"
				replace `V'=(`V'+`sh`i'')/`sc`i''
			}
		}
		if "`adj`i''"!="" {
			if "`adj`i''"!="no" {
				local a: di %9.0g (`adj`i''+`sh`i'')/`sc`i''
				local a=trim("`a'")
			}
			if "`adjust'"=="" {
				local comma
			}
			else local comma ,
			local adjust `adjust'`comma'`V':`a'
		}
		local i=`i'+1
	}
	if "`adjust'"!="" {
		local adjopt adjust(`adjust')
	}
	if `"`saving'"'!="" {
		tempfile tmp0 tmp
	}
	noi di as txt _n "Bootstrapping ..."
	local i=cond("`basedata'"=="", 1, 0) /* i=0 gives model for base data */
	local i0 `i'
	while `i'<=`replicates' {
		noi di `i', _cont
		preserve
		drop if `touse'==0
		if `i'>0 & "`resample'"!="noresample" {
			bsample
		}
		if `i'==1 & "`resample'"=="noresample" {
			/* impose random covariate order */
			local xordopt xorder(r)
		}
		if `"`saving'"'!="" {
			save `tmp0', replace
			cap drop i
			gen long i=`i'
			order i `yvar' `dead' `rhs' `keepalso'
			keep i `yvar' `dead' `rhs' `keepalso'
			if `i'==`i0' {
				lab var i "Boot replication"
				save `"`saving'"', replace
			}
			else save `tmp', replace
			use `tmp0', replace
		}
		capture mfp `cmd' `yvar' `RHS' [`weight'`exp'], /*
		 */ `options' noscaling `adjopt' `xordopt'
		local rc=_rc
		if `rc'==1 {
			noisily error 1
		} 
		if `rc'==198 {
			noisily di as err "warning: encountered syntax error in mfp"
		}
		if `rc'==0 {
			if `i'==`i0' {	/* initialise postfile */
/*
	Handle constant. If not in model, don't attempt to save it to file.
*/
				local hascons 0
				cap local B0=_b[_cons]
				if _rc==0 {
					local pb `pb' b0
					local hascons 1
				}
				tempname temp
				postfile `temp' i `pb' using `outfile', `replace' double
			}
			local posts
			tokenize `e(fp_fvl)'
			local j 1
			while `j'<=`nx' {
				if "`e(fp_k`j')'"=="." {
					local np 0	/* var not in model */
				}
				else {
					local np: word count `e(fp_k`j')'
/*
	Screen for collinearities indicated by a missing coefficient (b)
	with a non-missing power (p). Drop variable.
*/
					local k 1
					while `k'<=`np' {
						local p:word `k' of `e(fp_k`j')'
						cap local b=_b[``k'']
						if ("`p'"!="." & "`p'"!="") /*
						 */ & ("`b'"=="." | "`b'"=="") {
						 	local np 0
						}
						local k=`k'+1
					}
					local k 1
					while `k'<=`np' {
						local p:word `k' of `e(fp_k`j')'
						cap local b=_b[``k'']
						if _rc {
							local b .
							local p .
						}
						local posts `posts' (`p') (`b')
						local k=`k'+1
					}
				}
				if `np'<`np`j'' {
					local k=`np'+1
					while `k'<=`np`j'' {
						local posts `posts' (.) (.)
						local k=`k'+1
					}
				}
				mac shift `np'
				local j=`j'+1
			}
			if `hascons' {
				post `temp' (`i') `posts' (_b[_cons])
			}
			else 	post `temp' (`i') `posts'
			if `"`saving'"'!="" & `i'>`i0' {
				use `"`saving'"', replace
				append using `tmp'
				save `"`saving'"', replace
			}
			local i=`i'+1
		}
		else {
			local failed=`failed'+1
		}
		restore
	}
	postclose `temp'
/*
	Store shift, scale and adjust values as characteristics,
	and list of variables in model.
*/
	preserve
	use `outfile', clear
	local vl
	local i 1
	while `i'<=`nx' {
		char `xs`i''p1[shift] `sh`i''
		char `xs`i''p1[scale] `sc`i''
		char `xs`i''p1[adjust] `adj`i''
		local vl `vl' `v`i''
		local i=`i'+1
	}
	char _dta[pmb_data] mfpboot
	char _dta[pmb_vl] `vl'
	compress
	save `outfile', replace
}
di _n(2) as res `replicates' as txt " bootstrap replicates saved to `outfile'."
if `"`saving'"'!="" {
	di as res `replicates' as txt `" bootstrap samples saved to `saving'."'
}
if `failed'>0 {
	di as inp "[mfp failed at " as res `failed' /*
	 */ as inp " replicates which were discarded.]"
}
end

program define GetVL, sclass /* varlist [if|in|,|[weight]] */
macro drop MFP_*
if $MFpdist != 7 {
	gettoken tok 0 : 0
	unabbrev `tok'
	global MFP_dv "`s(varlist)'"
}

global MFP_cur		/* MFP_cur will contain full term list */
global MFP_n 0

gettoken tok : 0, parse(" ,[")
IfEndTrm "`tok'"
while `s(IsEndTrm)'==0 {
	gettoken tok 0 : 0, parse(" ,[")
	if substr("`tok'",1,1)=="(" {
		local list
		while substr("`tok'",-1,1)!=")" {
			if "`tok'"=="" {
				di as err "varlist invalid"
				exit 198
			}
			local list "`list' `tok'"
			gettoken tok 0 : 0, parse(" ,[")
		}
		local list "`list' `tok'"
		unabbrev `list'
		global MFP_n = $MFP_n + 1
		global MFP_$MFP_n "`s(varlist)'"
		global MFP_cur "$MFP_cur `s(varlist)'"
	}
	else {
		unabbrev `tok'
		local i 1
		local w : word 1 of `s(varlist)'
		while "`w'" != "" {
			global MFP_n = $MFP_n + 1
			global MFP_$MFP_n "`w'"
			local i = `i' + 1
			local w : word `i' of `s(varlist)'
		}
		global MFP_cur "$MFP_cur `s(varlist)'"
	}
	gettoken tok : 0, parse(" ,[")
	IfEndTrm "`tok'"
}
sret local nought `0'
end

program define IfEndTrm, sclass
sret local IsEndTrm 1
if "`1'"=="," | "`1'"=="in" | "`1'"=="if" | /*
*/ "`1'"=="" | "`1'"=="[" {
	exit
}
sret local IsEndTrm 0
end
