*! version 1.0.0 PR 19mar2008
* Heavily based on mfpint v 1.0.6. Incorporates elements of mfpint100.ado.
program define mfpi, rclass
version 10.0
gettoken cmd 0: 0, parse(" ,")
frac_chk `cmd' 
if `s(bad)' { 
	di as err "invalid or unrecognised command, `cmd'"
	exit 198
}
local dist `s(dist)'
/*
	!! Hidden option `adjbin' adjusts for treatment variable
	in adjustment model. Won't make much difference in randomised controlled
	trials, but may be important in observational data. To be discussed.

	Flexibility (flex()): 1 = least (S & R 2004, mfpint/mfpi.ado),
	2 = intermediate (original version of paper, mfpint100.ado), 
	3 = intermediate (allow different powers for main effect from interaction),
	4 = most (allow different powers in each group and for main effect).
*/
syntax [varlist(default=none)] [if] [in] [aw fw pw iw] , ///
 WITH(varname)[ SELect(string) ADDpowers(string) ALpha(string) ADJBin ADJust(varlist) ALL ///
 DEAD(varname) DETail DF(string) FLex(int 1) FP1(varlist) FP2(varlist) GENF(string) GENDiff(string) ///
 LINear(string) POWers(string) noSEdiff SHOwmodel ZERo(string) MFPopts(string) TOPcoded * ]
// `sediff' is now implied by gendiff(), unless the nosediff option is used.
if "`linear'`fp1'`fp2'"=="" {
	di as err "you must specify interaction term(s)"
	exit 198
}
* Store original lists of variables for interaction analysis with linear, FP1, FP2 models
local Linear `linear'
local Fp1 `fp1'
local Fp2 `fp2'
if "`powers'"=="" local powers -2 -1 -0.5 0 0.5 1 2 3
tokenize `powers' `addpowers'
local i 1
while "``i''"!="" {
	local p`i' ``i''
	local ++i
}
local np = `i'-1
if "`detail'"!="" {
	local noi noi
}
local powers powers(`powers' `addpowers')
local items 0	/* overall number of interactions considered */
if "`linear'"!="" {
	* Sort out bound partners
	local lsep "("
	local rsep ")"
	tokenize `linear', parse(" `lsep'`rsep'")
	local linear
	local lparen 0
	local cat
	local ncat 0
	while "`1'"!="" {
		if "`1'"=="`lsep'" {
			if `lparen' {
				noi di as err "unexpected `lsep' in linear()"
				exit 198
			}
			local lparen 1
		}
		else if "`1'"=="`rsep'" {
			if `lparen'==0 {
				noi di as err "unexpected `rsep' in linear()"
				exit 198
			}
			local ++ncat
			unab iz`ncat': `cat'
			local linear `linear' `iz`ncat''
			local ifp`ncat' 0	/* ifp=0 indicates linear term */
			local cat
			local lparen 0
		}
		else {
			if `lparen'==0 {
				local ++ncat
				unab iz`ncat': `1'
				local linear `linear' `iz`ncat''
				local ifp`ncat' 0
			}
			else local cat `cat' `1'
		}
		mac shift
	}
	if `lparen' {
		noi di as err "unexpected `rsep' in linear()"
		exit 198
	}
	local items `ncat'
}
if "`fp1'"!="" {
	local nfp1: word count `fp1'
	forvalues i = 1/`nfp1' {
		local ++items
		local iz`items': word `i' of `fp1'
		local ifp`items' 1
	}
}
if "`fp2'"!="" {
	local nfp2: word count `fp2'
	forvalues i = 1/`nfp2' {
		local ++items
		local iz`items': word `i' of `fp2'
		local ifp`items' 2
	}
}
quietly {
	if `dist'==7 {	/* stcox, streg */
		local xvars `varlist'
		local yvar
	}
	else gettoken yvar xvars: varlist
	marksample touse
	markout `touse' `varlist' `dead' `with' `linear' `fp1' `fp2' `adjust'
	frac_wgt "`exp'" `touse' "`weight'"
	local wgt `r(wgt)'				/* [`weight'`exp'] */
	if "`dead'"!="" {
		local options "`options' dead(`dead')"
	}
	if "`all'"=="" local ifuse if `touse'
	else local restrict restrict(if `touse')
	* Exclude `with' variable from xvars, if it was there
	local Xvars `xvars'
	local Xvars: list Xvars - with
	tempvar ic f tmp

	egen byte `ic' = group(`with') if `touse'
	* create dummies for levels of `ic'
	sum `ic' if `touse'
	local ndum = r(max)-1
	if `ndum'<1 {
		di as err "`with' must have at least two levels"
		exit 198
	}
	replace `ic' = `ic'-1	/* lowest level labelled 0 */
	if "`topcoded'"!="" replace `ic' = `ndum'-`ic'
	local Ic
	forvalues i=1/`ndum' {
		cap drop _Ic`i'
		gen byte _Ic`i' = (`ic'==`i') if `touse'
		local Ic `Ic' _Ic`i'
	}
	count if `touse'
	local nobs = r(N)
	local nxvar: word count `Xvars'
	if `nxvar'==0 {
		if "`select'"!="" & "`adjust'"!="" {
			noi di as txt "[select() ignored]"
		}
	}
	else {
		if "`select'"=="" {
			local select 1
		}
		* Find adjustment model for main covariates
		if "`alpha'"!="" {
			local Alpha alpha(`alpha')
		}
		* Force dummies from interaction variable into adjustment model
		if "`adjbin'"!="" {
			local adjbin `Ic'
			local select `select', `adjbin':1
		}
		if "`adjust'"!="" {
			if "`df'"!="" 	local Df df(`df',`adjust':1)
			else 		local Df df(`adjust':1)
			local select `select', `adjust':1
		}
		else {
			if "`df'"!="" 	local Df df(`df')
		}
		local Select select(`select')
		`noi' mfp `cmd' `yvar' `Xvars' `adjust' `adjbin' `wgt' if `touse', /*
		 */ `Alpha' `Df' `Select' `powers' `options' `mfpopts'
		if "`showmodel'"!="" {
			noi di as txt _n "Variables in adjustment model" _n "{hline 29}"
			if "`adjust'"!="" noi di as txt "[`adjust': linear]"
			if "`adjbin'"!="" noi di as txt "[`adjbin': `with']"
		}
		* Store details of model for selected variables and powers
		local nxf 0
		forvalues i = 1/`nxvar' {
			local p `e(fp_k`i')'
			local x `e(fp_x`i')'
			if "`showmodel'"!="" noi di as txt %10s "`x':" _cont
			if "`p'"!="." {
				if "`showmodel'"!="" noi di as txt " power(s) = " as res "`p'"
				local ++nxf
				local x`nxf' `x'
				local fp`nxf' `p'
				if "`p'"!="1" {
					fracgen `x' `p' `ifuse', replace name(_Ix`nxf') `restrict'
					local n`nxf' `r(names)'
				}
				else local n`nxf' `x'
				local fxvars `fxvars' `n`nxf''
			}
			else {
				if "`showmodel'"!="" noi di as txt " not selected"
			}
		}
	}
	local d3 79
	local flexmess = cond(`flex'==1, "(least flexible)", cond(`flex'==4, "(most flexible)", "(intermediate)"))
	noi di as txt _n "Interactions with `with' (" as res `nobs' ///
	 as txt " observations). Flex-`flex' model `flexmess'"
	noi di as txt _n "{hline `d3'}"
	noi di as txt "Var         Main        Interact     idf   Chi2     P     Deviance tdf   AIC"
	noi di as txt "{hline `d3'}"
	forvalues ni = 1/`items' {
		local z `iz`ni''
		local nz: word count `z'	/* `z' could be a varlist */
		local degree `ifp`ni''		/* 0, 1 or 2; 0=linear */
/*
	Remove members of z from adjustment model varlist
	(note that adjlin vars, if any, are not counted in nxvar)
*/
		local xvars
		if `nxvar'>0 {
			forvalues i = 1/`nz' {
				local zi: word `i' of `z'
				forvalues j = 1/`nxvar' {
					if "`zi'"!="`x`j''" local xvars `xvars' `n`j''
				}
			}
		}
/*
	Deal with FP case
*/
		if `degree'>0 {
			* Determine shift in z, if needed to avoid zeros.
			fracgen `z' 0 if `touse', nogen
			local shift = r(shift)
			local scale = r(scale)
			cap drop _Iz
			gen _Iz = (`z'+`shift')/`scale' `ifuse'
/*
	Create continuous z at each level, with 0 for other levels
*/
			local unvi	// names of untransformed z at each level
			forvalues i = 0/`ndum' {
				cap drop _Iz`i'
				gen _Iz`i' = cond(`ic'==`i', (`z'+`shift')/`scale', 0) `ifuse'
				local unvi `unvi' _Iz`i'
			}
/*
	Main-effects model.
	Determine powers for flex 1, 3 and 4 using fracpoly.
	Flex 2 uses powers from interaction model.
*/
			local dfmain = 2*`degree'+`ndum'
			if `flex'==1 | `flex'==3 | `flex'==4 {
				fracpoly `cmd' `yvar' _Iz `xvars' `adjust' `Ic' `wgt' if `touse', degree(`degree') `powers' `options'
				local powmain `e(fp_pwrs)'
				fracgen _Iz `powmain' `ifuse', replace noscaling `restrict'
				local v `r(names)'
			}
/*
	Interaction models
*/
			local dfint = `ndum' + cond(`flex'<4, (`ndum'+2)*`degree', (`ndum'+1)*2*`degree')
			if `flex'==1 {
/*
	Fit model with powers from main effect at all levels
*/
				local powint `powmain'
				local vi	// names of FP-transformed vars at each level
				forvalues i = 0/`ndum' {
					if "`powint'"!="1" {
						fracgen _Iz`i' `powint' `ifuse', zero replace noscaling `restrict'
						local v`i' `r(names)'
					}
					else local v`i' _Iz`i'
					local vi `vi' `v`i''
				}
			}
			else if `flex'==2 | `flex'==3 {
/*
	Determine interaction powers for flex 2 and 3.
	Force powers to be the same for all levels (= `powint').
*/
				local devbest 1e30
				forvalues j = 1/`np' {
					if `degree'==2 {
						forvalues k = `j'/`np' {
							local iint
							forvalues i = 0/`ndum' {
								fracgen _Iz`i' `p`j'' `p`k'' if `touse', replace zero
								local iint `iint' `r(names)'
							}
							`cmd' `yvar' `xvars' `adjust' `Ic' `iint' `wgt' if `touse', `options'
							local devint = -2*e(ll)
							if `devint'<`devbest' {
								local devbest `devint'
								local powint `p`j'' `p`k''
							}
						}
					}
					else {
						local iint
						forvalues i = 0/`ndum' {
							fracgen _Iz`i' `p`j'', replace zero
							local iint `iint' `r(names)'
						}
						`cmd' `yvar' `xvars' `adjust' `Ic' `iint' `wgt' if `touse', `options'
						local devint = -2*e(ll)
						if `devint'<`devbest' {
							local devbest `devint'
							local powint `p`j''
						}
					}
				}
				local vi	// names of FP-transformed vars at each level
				forvalues i=0/`ndum' {
					if "`powint'"!="1" {
						fracgen _Iz`i' `powint' `ifuse', zero replace noscaling `restrict'
						local v`i' `r(names)'
					}
					else local v`i' _Iz`i'
					local vi `vi' `v`i''
				}
			}
			else if `flex'==4 {
/*
	Use mfp to determine possibly different powers (`powint`i'') at each level.
*/
				if `degree'==2 local mfpdf df(1, `unvi':4)
				else local mfpdf df(1, `unvi':2)
				mfp `cmd' `yvar' `unvi' `xvars' `adjust' `Ic' `wgt' if `touse', ///
				 `mfpdf' alpha(1) select(1) zero(`unvi') `options'
				local vi	// names of FP-transformed vars at each level
				forvalues i = 0/`ndum' {
					local i1 = `i'+1
					local powint`i' `e(fp_k`i1')'	// estimated power(s) at each level
					if "`powint`i''"!="1" {
						fracgen _Iz`i' `powint`i'' `ifuse', zero replace noscaling `restrict'
						local v`i' `r(names)'
					}
					else local v`i' _Iz`i'
					local vi `vi' `v`i''
				}
			}
/* 
	Fit flex 2 main-effects model, using powers from interaction (in `powint')
*/
			if `flex'==2 {
				local powmain `powint'
				fracgen _Iz `powmain' `ifuse', replace noscaling `restrict'
				local v `r(names)'
			}
		}
		else {
/*
	Deal with linear case (includes binary and categoric).
*/
			local dfmain = `nz'+`ndum'
			local dfint = `dfmain'+`nz'*`ndum'
			local powmain Linear
			local powint Linear
			local vi
			forvalues i = 0/`ndum' {
				local v`i'
				forvalues j = 1/`nz' {
					local zj: word `j' of `z'
					cap drop _Iz`i'_`j'
					gen _Iz`i'_`j' = cond(`ic'==`i', `zj', 0) `ifuse'
					local v`i' `v`i'' _Iz`i'_`j'
				}
				local vi `vi' `v`i''
			}
			* Main-effect covariate(s)
			local v `z'
		}	/* end of linear case */
/*
	Fit main-effects model and interaction model (in that order).
	`v'  is varlist for main effect of covariate(s) z,
	`vi' is varlist for interaction and main effect of covariate(s) z,
	`Ic' is varlist for main effect of treatment.
*/
		`cmd' `yvar' `xvars' `adjust' `Ic' `v' `wgt' if `touse', `options'
		local devmain = -2*e(ll)
		`cmd' `yvar' `xvars' `adjust' `Ic' `vi' `wgt' if `touse', `options'
		local devint = -2*e(ll)
/*
	Test interaction
*/
		local totaldf`degree' `dfint'
		local chi = `devmain'-`devint'
		// df vary according to flex
		local dfd_default = `dfint'-`dfmain'
		if `flex'==2 local dfd = `dfd_default'*(`degree'==0) + 2*(`degree'==1) + 4*(`degree'==2)	// perm test results
		else local dfd `dfd_default'
		local P = chiprob(`dfd',`chi')
/*
		Store details of test statistic for this variable.
		(Note: only details of last variable in list will be finally stored.)
*/
		local dfd`degree' `dfd'
		local chi2`degree' `chi'
		local P`degree' `P'
		local dev`degree' `devint'
		local aic`degree' = `devint'+2*`totaldf`degree''

		if length("`z'")>10 local showz = abbrev("`z'",10)
		else local showz `z'
		if `degree'==0 {
			local term1 "Linear"
			local term2 "Linear"
		}
		else {
			local term1 "FP`degree'(`powmain')"
			if `flex'==4 {
				local starmess "* possibly more than one set of FP powers, shown only for 1st dummy variable"
				local term2 "FP`degree'(`powint0')*"
			}
			else local term2 "FP`degree'(`powint')"
		}
		noi di as txt %-12s "`showz'" %-12s "`term1'" %-12s "`term2'" as res /*
		 */ %3s "`dfd'" %8.2f `chi' %9.4f `P' %10.3f `devint' %3.0f `totaldf`degree'' %10.3f `aic`degree''
		if "`genf'`gendiff'"!="" {
			/* Note that correct prediction depends on fitting
			   interaction model last in the lines above this.
			   Create full-sample versions of FP of z for prediction.
			*/
			forvalues i=0/`ndum' {
				if `degree'>0 {
					if `flex'==4 local pwr `powint`i''
					else local pwr `powint'
					if "`pwr'"!="1" {
						fracgen _Iz `pwr' `ifuse', zero replace noscaling name(_Iz`i') `restrict'
					}
					else replace _Iz`i' = _Iz
				}
				else {
					forvalues j = 1/`nz' {
						local zj: word `j' of `z'
						replace _Iz`i'_`j'=`zj'
					}
				}
			}
			if "`cmd'"=="poisson" {
				* need to specify eqname here, for xpredict
				local y `e(depvar)'
				local eq eq(`y')
			}
			if "`genf'"!="" {
				forvalues i = 0/`ndum' {
					*noi confirm new var `genf'`ni'_`i'
					* Predict function at this level
					cap drop `f'
					xpredict `f' `ifuse', with(`v`i'') `eq'
/*
					* (no longer) Centre function on level 0
					if `i'==0 {
						sum `f' if `ic'==0 & `touse'==1
						local f0mean=r(mean)
					}
					else replace `f'=`f'+_b[_Ic`i']
*/
					if `i'>0 replace `f' = `f'+_b[_Ic`i']
					cap drop `genf'`ni'_`i'
					rename `f' `genf'`ni'_`i'
					if `degree'==0 lab var `genf'`ni'_`i' "f(`z'), level `i'" 
					else lab var `genf'`ni'_`i' "FP`degree'(`z'), level `i'"
				}
				/* Following code used to work OK, but SEs depend on
				   centering of covariates, which is not handled.
				   Don't want to get into that issue here!
				if "`se'"!="" {
					drop `f0' `f1'
					xpredict `f0' if `touse', with(`v0') stdp
					xpredict `f1' if `touse', with(`v1' _Ic) stdp
					cap drop `genf's0
					cap drop `genf's1
					rename `f0' `genf's0
					rename `f1' `genf's1
					lab var `genf's0 "SE(FP`degree'(`z')), group 0"
					lab var `genf's1 "SE(FP`degree'(`z')), group 1"
				}
				*/
			}
			if "`gendiff'"!="" {
				local Z = -invnormal((100-c(level))/200)
				* Predict f`i'-f0 via suitable contrast, L
				if `degree'==2 {
					local L 1 1 1 -1 -1
				}
				else if `degree'==1 {
					local L 1 1 -1
				}
				else {
					local L1
					local L2
					forvalues j = 1/`nz' {
						local L1 `L1' 1
						local L2 `L2' -1
					}
					local L 1 `L1' `L2'
				}
				forvalues i = 1/`ndum' {
					* noi confirm new var `gendiff'`ni'_`i'
					rename _Ic`i' `tmp'
					gen byte _Ic`i' = 1 `ifuse'
					cap drop `f'
					xpredict `f' `ifuse', with(_Ic`i' `v`i'' `v0') `eq' a(`L')
					if `degree'==0 lab var `f' "Diff in f(`z'), level `i'-level 0"
					else lab var `f' "Diff in FP`degree'(`z'), level `i'-level 0"
					cap drop `gendiff'`ni'_`i'
					rename `f' `gendiff'`ni'_`i'
					* SE(diff) is independent of covariate centering
					if "`sediff'"!="nosediff" {
						xpredict `f' `ifuse', with(_Ic`i' `v`i'' `v0') `eq' a(`L') stdp
						if `degree'==0 {
							lab var `f' "SE(Diff in f(`z'), level `i'-level 0)"
						}
						else lab var `f' "SE(Diff in FP`degree'(`z'), level `i'-level 0)"
						cap drop `gendiff'`ni'lb_`i'
						cap drop `gendiff'`ni'ub_`i'
						gen `gendiff'`ni'lb_`i' = `gendiff'`ni'_`i'-`Z'*`f'
						gen `gendiff'`ni'ub_`i' = `gendiff'`ni'_`i'+`Z'*`f'
						cap drop `gendiff'`ni's_`i'
						rename `f' `gendiff'`ni's_`i'
					}
					drop _Ic`i'
					rename `tmp' _Ic`i'
				}
			}
		}
	}
	noi di as txt "{hline `d3'}" ///
	 _n "idf = interaction degrees of freedom; tdf = total model degrees of freedom"
	if "`starmess'"!="" noi di as txt "`starmess'"
	* Tidy up (ruthlessly)
	cap drop _IC*
	cap drop _Ic*
	cap drop _Ix*
	cap drop _Iz*
}
return local if `if'
return local in `in'
return local varlist `varlist'
return local dead `dead'
return local with `with'
/*
	Store details of interaction test(s) for final variable
	in each list (linear, fp1, fp2)
*/
if "`Linear'"!="" {
	return local Linear `Linear'
	return scalar chi2lin = `chi20'
	return scalar Plin = `P0'
	return scalar devlin = `dev0'
	return scalar aiclin = `aic0'
	return scalar totdflin = `totaldf0'
}
if "`Fp1'"!="" {
	return local Fp1 `Fp1'
	return scalar chi2fp1 = `chi21'
	return scalar Pfp1 = `P1'
	return scalar devfp1 = `dev1'
	return scalar aicfp1 = `aic1'
	return scalar totdffp1 = `totaldf1'
}
if "`Fp2'"!="" {
	return local Fp2 `Fp2'
	return scalar chi2fp2 = `chi22'
	return scalar Pfp2 = `P2'
	return scalar devfp2 = `dev2'
	return scalar aicfp2 = `aic2'
	return scalar totdffp2 = `totaldf2'
}
return local adjust `adjust'
return local z `z'
if "`z'"!="" {
	return local powmain `powmain'
	if `flex'==4 {
		forvalues i = 0/`ndum' {
			return local powint`i' `powint`i''
		}
	}
	else return local powint `powint'
}
return scalar nxvar = `nxvar'		
if `nxvar'>0 {
	* save FP adjustment model details
	forvalues i = 1/`nxf' {
		return local x`i' `x`i''
		return local power`i' `fp`i''
	}
	return scalar nxf = `nxf'
}
end

*! version 1.0.3 PR 28apr2006
program define xpredict
version 6
syntax newvarname [if] [in], WITH(varlist) [ CONStant A(numlist) DOUble EQ(string) mi * ]
if "`mi'"!="" {
	confirm var _j
}
tempname tmp b V b2 V2
matrix `b'=e(b)
matrix `V'=e(V)
if "`constan'"!="" {
	local with `with' _cons
}
if "`eq'"!="" {
	* include equation name with variable(s)
	tokenize `with'
	local with
	while "`1'"!="" {
		local with `with' `eq':`1'
		mac shift
	}
}
matselrc `b' `b2', row(1) col(`with')
matselrc `V' `V2', row(`with') col(`with')
local nc=colsof(`b2')
if "`a'"!="" {
	local na: word count `a'
	if `na'!=`nc' {
		di in red "wrong number of elements in a(), should be `nc'"
		exit 198
	}
	* construct matrix from `a' by hand
	tempname A
	matrix `A'=J(1,`nc',0)
	tokenize `a'
	local i 1
	while `i'<=`nc' {
		matrix `A'[1,`i']=``i''
		local i=`i'+1
	}
	local cn: colnames `b2'
	* Linear combination of elements in b2 with a.
	* Multiply A by b2 elementwise and overwrite b2.
	qui matewm `A' `b2' `b2'
	matrix colnames `b2'=`cn'
	* Form outer product of A with itself then multiply by V2 elementwise,
	* overwriting V2.
	matrix `A'=`A''*`A'
	qui matewm `A' `V2' `V2'
	matrix colnames `V2'=`cn'
	matrix rownames `V2'=`cn'
}
estimates hold `tmp'
capture estimates post `b2' `V2'
local rc=_rc
if `rc' {
	estimates unhold `tmp'
	error `rc'
}
qui if "`mi'"!="" {
	* Partial prediction for all obs, then average over imputations
	sum _j, meanonly
	local m=r(max)
	tempvar f
	if `"`if'"'=="" {
		local If if _j>0
	}
	else local If `if' & _j>0
	capture predict `double' `f' `If' `in', xb
	local rc=_rc
	estimates unhold `tmp'
	if `rc' {
		error `rc'
	}
	sort _i _j
	by _i: gen `double' `varlist'=sum(`f')/`m' if _j>0
	by _i: replace `varlist'=`varlist'[_N] if _n<_N & _j>0
}
else {
	capture predict `double' `varlist' `if' `in', `options'
	local rc=_rc
	estimates unhold `tmp'
	if `rc' {
		error `rc'
	}
}
end

*! NJC 1.1.0 20 Apr 2000  (STB-56: dm79)
program def matselrc
* NJC 1.0.0 14 Oct 1999 
        version 6.0
        gettoken m1 0 : 0, parse(" ,")
        gettoken m2 0 : 0, parse(" ,") 
	
	if "`m1'" == "," | "`m2'" == "," | "`m1'" == "" | "`m2'" == "" { 
		di in r "must name two matrices" 
		exit 198
	} 
	
        syntax , [ Row(str) Col(str) Names ]
        if "`row'`col'" == "" {
                di in r "nothing to do"
                exit 198
        }

        tempname A B 
        mat `A' = `m1' /* this will fail if `matname' not a matrix */
	local cols = colsof(`A') 
	local rows = rowsof(`A') 

        if "`col'" != "" {
		if "`names'" != "" { local colnum 1 } 
		else { 
	                capture numlist "`col'", int r(>0 <=`cols')
			if _rc == 0 { local col "`r(numlist)'" } 
                	else if _rc != 121 { 
				local rc = _rc 
				error `rc' 
			} 	
			local colnum = _rc == 0 
		}	
		/* colnum = 1 for numbers, 0 for names */ 

		tokenize `col' 
		local ncols : word count `col' 
		if `colnum' { 
			mat `B' = `A'[1..., `1'] 
			local j = 2 
			while `j' <= `ncols' { 
                		mat `B' = `B' , `A'[1..., ``j'']
				local j = `j' + 1 
			} 	
		} 
		else {
			mat `B' = `A'[1..., "`1'"] 
			local j = 2 
			while `j' <= `ncols' { 
                		mat `B' = `B' , `A'[1..., "``j''"]
				local j = `j' + 1 
			} 	
		} 
		mat `A' = `B' 	
		local cols = colsof(`A')  		
        }
	
	if "`row'" != "" {
		if "`names'" != "" { local rownum 0 } 
		else { 
	                capture numlist "`row'", int r(>0 <=`rows')
			if _rc == 0 { local row "`r(numlist)'" } 
                	else if _rc != 121 { 
				local rc = _rc 
				error `rc' 
			} 	
			local rownum = _rc == 0   
		} 	
		/* rownum = 1 for numbers, 0 for names */ 

		tokenize `row' 
		local nrows : word count `row' 
		if `rownum' { 
			mat `B' = `A'[`1', 1...] 
			local j = 2 
			while `j' <= `nrows' { 
                		mat `B' = `B' \ `A'[``j'', 1...]
				local j = `j' + 1 
			} 	
		} 
		else {
			mat `B' = `A'["`1'", 1...] 
			local j = 2 
			while `j' <= `nrows'  { 
                		mat `B' = `B' \ `A'["``j''", 1...]
				local j = `j' + 1 
			} 	
		} 
		mat `A' = `B' 	
        }
	
        mat `m2' = `A'
end
program define matewm
*! 1.0.1 NJC 15 June 1999 STB-50 dm59
* 1.0.0  NJC 21 July 1998
    version 5.0
    parse "`*'", parse(" ,")
    if "`3'" == "" | "`3'" == "," {
        di in r "invalid syntax"
        exit 198
    }

    matchk `1'
    local A "`1'"
    matchk `2'
    local B "`2'"
    matcfa `A' `B'
    local nr = rowsof(matrix(`A'))
    local nc = colsof(matrix(`A'))
    local C "`3'"
    mac shift 3
    local options "Format(str)"
    parse "`*'"

    tempname D
    mat `D' = J(`nr',`nc',1)
    local i 1
    while `i' <= `nr' {
        local j 1
        while `j' <= `nc' {
            mat `D'[`i',`j'] = `A'[`i',`j'] * `B'[`i',`j']
            local j = `j' + 1
        }
        local i = `i' + 1
    }

    if "`format'" == "" { local format "%9.3f" }
    mat `C' = `D' /* allows overwriting of either `A' or `B' */
    mat li `C', format(`format')
end
*! 1.0.0 NJC 19 July 1998    STB-50 dm69
program def matcfa
* matrices conformable for addition?
* matcfa `1' `2'
    version 5.0
    if "`1'" == "" | "`2'" == "" | "`3'" != "" {
        di in r "invalid syntax"
        exit 198
    }
    tempname C
    mat `C' = `1' + `2'
end
*! 1.0.0 NJC 5 July 1998    STB-50 dm69
program def matchk
* matrix?
* matchk `1'
    version 5.0
    if "`1'" == "" | "`2'" != "" {
        di in r "invalid syntax"
        exit 198
    }
    tempname C
    mat `C' = `1'
end
