*! version 1.0.0 30apr2007
program define mvss, eclass sortpreserve
version 8
if "`1'" == "" | "`1'"=="," {
	if "`e(fp_cmd2)'"!="mfp" {
		error 301
	}
	FracRep "GAM" "  df  " "e.d.f."
	exit
}
gettoken cmd 0 : 0
cmd_chk `cmd' 
if `s(bad)' {
	di as err "invalid or unrecognized command, `cmd'"
	exit 198
}
/*
	dist=0 (regress), 1 (logit), 2 (poisson), 3 (cox), 6(gamma), 7(stcox).
	Only 0, 1, 3 and 7 are known to work so far. No non-canonical links implemented.
*/
local dist `s(dist)'
local normal = (`dist'==0)
global MFpdist `dist'

* Disentangle
GetVL `0'
local 0 `s(nought)'

syntax [if] [in] [aw fw pw iw] , [ AIC ALpha(str) DF(str) DFDefault(int 4) ///
 CYCles(int 20) DEAD(str) SELect(str) SEQuential noTESTLinear XOrder(str) * ]
if `"`options'"'!="" local gamopts gamopts(`options')
frac_cox "`dead'" `dist'
/*
	Process options
*/

if "`dead'"!="" local regopt "`regopt' dead(`dead')"
if "`aic'"!="" { /* aic selection for vars and functions */
	if "`alpha'`select'"!="" {
		noi di as err "alpha() and select() invalid with aic"
		exit 198
	}
	local alpha -1
	local select -1
}/*
	Check for missing values in lhs, rhs and model vars.
*/
tempvar touse
quietly {
	marksample touse
	markout `touse' $MFP_cur `dead'
	frac_wgt "`exp'" `touse' "`weight'"
	local wgt `r(wgt)'				/* [`weight'`exp'] */
	count if `touse'
	local nobs = r(N)
}
/*
	Detect collinearity among covariates, and fail if found.
*/
local ncur: word count $MFP_cur
_rmcoll $MFP_cur `if' `in' [`weight' `exp'], `constant'
local ncur2: word count `r(varlist)'
if `ncur2'<`ncur' {
	local ncoll=`ncur'-`ncur2'
	if `ncoll'>1 {
		local s ies
	}
	else local s y
	di as err `ncoll' " collinearit`s' detected among covariates"
	exit 198
}
/*
	Rearrange order of variables in varlist
*/
if "`xorder'"=="" {
	local xorder "+"
}
/*
	Apply fracord to order variables by P-value
*/
FracOrd `wgt' if `touse', order(`xorder') `regopt' cmd(`cmd') `gamopts'
local nx $MFP_n	/* number of clusters, <= number of predictors */
local lhs $MFP_dv
forvalues i=1/`nx' {
/*
	Store original order and reverse order
	of each RHS variable/variable set
*/
	local r`i' `s(ant`i')'
}
/*
	Initialisation.
*/
forvalues i=1/`nx' {
	local x ${MFP_`i'}
	local nx`i': word count `x'
	local alp`i' .05	/* default FP selection level */
	local h`i' `x'		/* names of H(xvars) 	*/
	local n`i' `x'		/* names of xvars 	*/
	local po`i' 1 		/* to be final power	*/
	local sel`i' 1		/* default var selection level */
}
/*
	Set up degrees of freedom for each variable
*/
if "`df'"!="" {
	FracDis "`df'" df 1 .
	forvalues i=1/`nx' {
		if "${S_`i'}"!="" local df`i' ${S_`i'}
	}
}
/*
	Assign default df for vars not so far accounted for.
	Give 1 df if 2-3 distinct values, 2 df for 4-5 values,
	dfdefault df for >=6 values.
*/
forvalues i=1/`nx' {
	if `nx`i''>1 /* over-ride all suggestions that df>1 for grouped vars */ local df`i' 1
	else {
		if "`df`i''"=="" {
			qui inspect ${MFP_`i'} if `touse'
			local u = r(N_unique)
			if `u'<=3 local df`i' 1
			else if `u'<=5 local df`i' = min(2,`dfdefault')
			else local df`i' `dfdefault'
		}
	}
}
/*
	Set up selection level (alpha) for each variable
*/
if "`alpha'"!="" {
	FracDis "`alpha'" alpha -1 1
	forvalues i=1/`nx' {
		if "${S_`i'}"!="" {
			local alp`i' ${S_`i'}
			if `alp`i''<0 {
				local alp`i' -1  /* AIC */
			}
		}
	}
}
/*
	Set up selection level (select) for each variable
*/
if "`select'"!="" {
	FracDis "`select'" select -1 1
	forvalues i=1/`nx' {
		if "${S_`i'}"!="" {
			local sel`i' ${S_`i'}
			if `sel`i''<0 local sel`i' -1  /* AIC */
		}
	}
}
/*
	Rationalise select() and alpha() in cases of aic
*/
forvalues i=1/`nx' {
	if `sel`i''==-1 & `alp`i''!=1 local alp`i' -1
	if `alp`i''==-1 & `sel`i''!=1 local sel`i' -1
}
/*
	Build gam model.
	`r*' macros present predictors according to FracOrd ordering,
	e.g. i=1, r`i'=3 means most sig predictor is third in user's xvarlist.
*/

local it 0
local initial 1
local stable 0 /* convergence flag */
while !`stable' & `it'<=`cycles' {
	local it = `it'+1
	local pwrs
	local rhs1
	local stable 1 /* later changed to 0 if any power or status changes */
	local lastch 0 /* becomes index of last var which changed status */
	forvalues i=1/`nx' {
		local r `r`i''
		local ni `n`r''
		local dfi df(`df`r'')
/*
	Build up RHS2 from the i+1th var to the end 
*/
		local rhs2
		local j `i'
		while `j'<`nx' {
			local j = `j'+1
			local rhs2 `rhs2' `h`r`j'''
		}
		if `initial' {
			if "`rhs2'"!="" local fixed "base(`rhs2')"
			else local fixed
			qui FracSel `cmd' `lhs' `ni' `wgt' if `touse', /*
			 */ df(1) `fixed' select(1) `regopt' `gamopts'
			local dev=r(dev)
			di in gr _n /*
			 */ "Deviance for model with all terms " /*
			 */ "untransformed = " in ye %9.3f r(dev) in gr ", " /*
			 */ in ye `nobs' in gr " observations"
		}
		if "`rhs1'`rhs2'"!="" local fixed "base(`rhs1' `rhs2')"
		else local fixed
/*
	Vars with df(1) are straight-line
*/
		local pvalopt "alpha(`alp`r'') select(`sel`r'')"
		if `i'==1 {
			di
		}
		if `df`r''==1 local fpo
		else local fpo `fpopt'
		if `df`r''==1 & `sel`r''==1 {	/* var is included anyway */
			local rhs1 `rhs1' `h`r''
			di in gr "[`ni' included with 1 df in model]" _n
		}
		else {
			if "`stub`r''"!="" {
				local n name(`stub`r'')
			}
			else local n
local n // !! temporary fix. n was something beginning with I. 
*noi di `"FracSel `cmd' `lhs' `ni' `wgt' if `touse', `dfi' `pw' `zero`r'' `catz`r'' `fixed' `h' `fpo' `regopt' `pvalopt' `n' `sequential' `testlinear'"'
			FracSel `cmd' `lhs' `ni' `wgt' if `touse', `dfi' /*
			 */ `fixed' `h' `fpo' `regopt' `pvalopt' `n' `testlinear' `gamopts'
			local h`r' `r(rhs)'
			local dev=r(dev)
			local p `r(pwrs)'
			if "`p'"!="`po`r''" {
				if `nx'>1 {
					local stable 0
				}
				local po`r' "`p'"
				local lastch `i'
			}
			if "`pwrs'"=="" {
				local pwrs "`p'"
			}
			else local pwrs "`pwrs',`p'"
			if "`h`r''"!="" {
				local rhs1 `rhs1' `h`r''
			}
		}
		if `initial' {
			local h "nohead"
			local initial 0
		}
	}
	if `lastch'==1 {
		local stable 1
	} /* 1 change only, at i=1 */
	if !`stable' {
		di in smcl in gr "{hline 70}" _n "End of Cycle " in ye `it' in gr /*
		 */ ": deviance =   " in ye %9.3f `dev' _n in gr "{hline 70}"
	}
}
if `nx'>1 {
	local s
	if `it'!=1 {
		local s "s"
	}
	if !`stable' {
		di _n in gr "No convergence" _cont
	}
	else di _n in gr /*
	 */ "GAM fitting algorithm converged" _cont
	di in gr " after " in ye `it' in gr " cycle`s'."
}
/*
	Store results
*/

local finalvl	// predictors in final model
local finaldf	// df for each predictor
forvalues i=1/`nx' {
	local p `po`i''
	local x `n`i''
	if trim("`p'")!="." {
		local finalvl `finalvl' `x'
		if `p'>1 local finaldf `finaldf' `x':`p',
	}
/*
	Save stuff for FracRep to display
*/
	local fd`i' `p'
	local id`i' `df`i''

	// Tidy up
	foreach thing in e r s {
		cap drop `thing'_`x'
		cap drop `thing'_s_`x'
	}
}
/*
	Estimate final (conditionally linear) model.
*/
if `dist'==0 local family gaussian
else if `dist'==1 local family binomial
else local family `cmd'
quietly if "`finalvl'"!="" {
	gam `lhs' `finalvl' `wgt' if `touse', df(`finaldf') `regopt' `options' family(`family')
}	
else {
	`cmd' `lhs' `wgt' if `touse', `regopt'
}
local dev = e(dev)
/*
	Store results for each predictor
	(requires expansion if grouped variables present)
*/
global S_1 `finalvl'
global S_2 `dev'
local nx2 0	/* number of predictors after expansion of groups (if any) */
forvalues i=1/`nx' {
	ereturn scalar Fp_fd`i'=`fd`i''	/* final degrees of freedom */
	ereturn scalar Fp_id`i'=`id`i''	/* initial degrees of freedom */
	ereturn scalar Fp_al`i'=`alp`i''	/* FP selection level */
	ereturn scalar Fp_se`i'=`sel`i''	/* var selection levl */
	ereturn local Fp_k`i' `po`i'' 	/* "powers" (df) for ith predictor */

	tokenize `n`i''
	while "`1'"!="" {
		local nx2=`nx2'+1
		ereturn local fp_x`nx2' `1'		/* name of ith predictor in user order */
		ereturn local fp_k`nx2' `po`i'' 	/* "powers" for ith predictor */
		mac shift
	}
}
ereturn scalar fp_dist=`dist'
ereturn local fp_wgt `weight'
ereturn local fp_exp `exp'
ereturn local fp_depv `lhs'
if `dist'==7 {
	ereturn local fp_depv _t
}
ereturn scalar fp_dev=`dev'
ereturn local fp_rhs	/* deliberately blank for consistency with fracpoly */
ereturn local fp_opts `regopt'
ereturn local fp_fvl `finalvl'
ereturn scalar fp_nx=`nx2'
ereturn local fp_t1t "GAM"
FracRep "GAM" "  df  " "e.d.f."

ereturn local fp_cmd "fracpoly"
ereturn local fp_cmd2 "mfp"

end

program define GetVL, sclass /* varlist [if|in|,|[weight]] */
macro drop MFP_*
if $MFpdist != 7 {
	gettoken tok 0 : 0
	unabbrev `tok'
	global MFP_dv "`s(varlist)'"
}

global MFP_cur		/* MFP_cur will contain full term list */
global MFP_n 0

gettoken tok : 0, parse(" ,[")
IfEndTrm "`tok'"
while `s(IsEndTrm)'==0 {
	gettoken tok 0 : 0, parse(" ,[")
	if substr("`tok'",1,1)=="(" {
		local list
		while substr("`tok'",-1,1)!=")" {
			if "`tok'"=="" {
				di as err "varlist invalid"
				exit 198
			}
			local list "`list' `tok'"
			gettoken tok 0 : 0, parse(" ,[")
		}
		local list "`list' `tok'"
		unabbrev `list'
		global MFP_n = $MFP_n + 1
		global MFP_$MFP_n "`s(varlist)'"
		global MFP_cur "$MFP_cur `s(varlist)'"
	}
	else {
		unabbrev `tok'
		local i 1
		local w : word 1 of `s(varlist)'
		while "`w'" != "" {
			global MFP_n = $MFP_n + 1
			global MFP_$MFP_n "`w'"
			local i = `i' + 1
			local w : word `i' of `s(varlist)'
		}
		global MFP_cur "$MFP_cur `s(varlist)'"
	}
	gettoken tok : 0, parse(" ,[")
	IfEndTrm "`tok'"
}
sret local nought `0'
end

program define IfEndTrm, sclass
sret local IsEndTrm 1
if "`1'"=="," | "`1'"=="in" | "`1'"=="if" | /*
*/ "`1'"=="" | "`1'"=="[" {
	exit
}
sret local IsEndTrm 0
end

program define FracOrd, sclass
version 8
sret clear
syntax [if] [in] [aw fw pw iw] [, CMd(string) ORDer(string) GAMopts(string) * ]
if "`cmd'"=="" {
	local cmd "regress"
}
if "`order'"=="" {
	di as err "order() must be specified"
	exit 198
}
local order=substr("`order'",1,1)
if "`order'"!="+" &"`order'"!="-" &"`order'"!="r" &"`order'"!="n" {
	di as err "invalid order()"
	exit 198
}
quietly {
	local nx $MFP_n
	if "`order'"=="n" {
		* variable order as given
		local i 1
		while `i'<=`nx' {
			local r`i' `i'
			local i=`i'+1
		}
	}
	else {
		if "`order'"=="+" | "`order'"=="-" {
			local dist $MFpdist
			if `dist'==0 		local family gaussian
			else if `dist'==1 	local family binomial
			else 				local family `cmd'
* noi di in red `"gam $MFP_dv $MFP_cur `if' `in' [`weight' `exp'], family(`family') `options' `gamopts'"'
			gam $MFP_dv $MFP_cur `if' `in' [`weight' `exp'], family(`family') `options' `gamopts'
		}
		tempvar c n
		tempname p dfnum dfres stat
		gen `c'=.
		gen int `n'=_n in 1/`nx'
		if "`order'"=="+" | "`order'"=="-" {
			forvalues i=1/`nx' {
				local n`i' ${MFP_`i'}
				capture test `n`i''	/* could comprise >1 variable */
				local rc=_rc
				if `rc'!=0 {
					noi di as err "could not test ${MFP_`i'}---collinearity?"
					exit 1001
				}
				scalar `p'=r(p)
				if missing(`p') scalar `p'=0
				if "`order'"=="-" { /* reducing P-value */
					replace `c'=-`p' in `i'
				}
				else replace `c'=`p' in `i'
			}
		}
		if "`order'"=="r" {
			replace `c'=uniform() in 1/`nx'
		}
		sort `c'
		forvalues i=1/`nx' {
/*
	Store positions of sorted predictors in user's list
*/
			local j 0
			while `j'<`nx' {
				local ++j
				if `i'==`n'[`j'] {
					local r`j' `i'
					local j `nx'
				}
			}
		}
	}
}
/*
	Store original positions of variables in ant1, ant2, ...
*/
forvalues i=1/`nx' {
	sret local ant`i' `r`i''
}
sret local names `names'
end

program define FracSel, rclass
version 8
gettoken cmd 0 : 0
local dist $MFpdist
local normal = (`dist'==0)
cmd_chk `cmd' 
if `s(bad)' {
	di as err "invalid or unrecognised command, `cmd'"
	exit 198
}
if `dist' != 7 {
	local vv varlist(min=2) 
}
else {
	local vv varlist(min=1)
}
if `normal' 		local family gaussian
else if `dist'==1 	local family binomial
else 				local family `cmd'
// `*' are usual regression options, gamopts are link(), etc.
syntax `vv' [if] [in] [aw fw pw iw] [, /*
*/ ALpha(real .05) SELect(real .05) noHEad DF(int -1) /*
*/ BAse(string) noTESTLinear GAMopts(string) * ]
/*
	18apr07
	1. Need to allow link() option for -gam- and -testvars-.
	2. Need to sort out dealing with gamma and poisson cases (gamma could be just glm, fam(gamma))
	3. Use of frac_dv in testvars is clunky. Can just use -2*e(ll) since there are no special cases.
	4. Don't need mnlnwt.
*/
if `df'==0 | `df'<-1 {
	di as err "invalid df"
	exit 198
}
local omit = (`select'<1)
local aic = (`select'==-1 | `alpha'==-1)
if `df'==-1 local df 4
local m = (`df'>1) // m=0 => linear

marksample touse
qui frac_wgt `"`exp'"' `touse' `"`weight'"'
local mnlnwt = r(mnlnwt) // mean log normalized weights
local weight `r(wgt)'

tokenize `varlist'
if $MFpdist != 7 {
	local lhs `1'
	local n `2'
	mac shift 2
}
else {
	local lhs
	local n `1'
	mac shift 1
}
local nn `*'
if "`head'"=="" {
	di in smcl in gr "Variable" _col(14) "Model" _col(20) "(vs.)" /*
	 */ _col(28) "Deviance" _col(38) "Dev diff.   P      e.d.f.   (vs.)"
	di in smcl "{hline 70}"
}
local vname `n' `nn'
if length("`vname'")>12 {
	local vname=substr("`vname'",1,9)+"..."
}
local pwrs 1
if "`nn'"!="" | `df'==1 {
	* test linear for single or group of predictors, adjusting for base
	local pwrs2 .
	if "`base'"!="" local base base(`base')
	local n `n' `nn'
	qui TestVars `cmd' `lhs' `n' `if' `in' `weight', `base' `options' `gamopts'
	local P = r(P)
	local dev1 = r(dev1)
	local dev0 = r(dev0)
	local d = r(devdif)
	local vs1 null
	local vs lin.
	local dfirst `dev0'
	if `aic'==0 {
		if `P'<=`select' {
			local star *
			local dev `dev1'
		}
		else {
			local star
			local n
			local dev `dev0'
		}
	}
	else {	/* select by AIC */
		local nnn: word count `n'	/* no. of vars being tested */
		if (`dev1'+`nnn'+`nnn')<`dev0' {
			local star *
			local dev `dev1'
		}
		else {
			local star
			local n
			local dev `dev0'
		}
	}
	di in smcl in gr "`vname'" _col(14) "`vs1'" /*
	 */ _col(21) "`vs'" _col(27) %9.3f in ye `dfirst' /*
	 */ _col(38) %8.3f `d' %7.3f `P' "`star'" /*
	 */ _col(57) "`pwrs2'" _col(67) "`pwrs'"
	if "`n'"=="" {
		local pwrs .
	}
	di in gr _col(14) "Final" _col(27) %9.3f in ye `dev' _col(57) "`pwrs'"
	di
	ret local rhs `n'
	ret local pwrs `pwrs'
	ret scalar dev = `dev'
	exit
}

local xvar `n'

// Compute null model and linear model deviances
if "`base'"!="" {
	local Base base(`base')
	local Df df(1, `n':`df')
}
else local Df df(`df')
* noi di in red "about to run testvars for null and lin devs"
qui TestVars `cmd' `lhs' `n' `if' `in' `weight', `Base' `options' `gamopts'
local dev1 = r(dev1)
local dev0 = r(dev0)

// Fit gam with rest of vars as given from previous cycles
 * noi di in red `"proper gam: gam `lhs' `n' `base' `if' `in' `weight', `Df' family(`family') `options' `gamopts'"'
capture gam `lhs' `n' `base' `if' `in' `weight', `Df' family(`family') `options' `gamopts'
 * noi di in red "gamdev = " e(dev)
local rc = _rc
if `rc' {
	noi di as err "failure encountered when estimating gam model"
	noi di as err `"gam `lhs' `n' `base' `if' `in' `weight', df(1, `n':`df') family(`family') `options' `gamopts'"'
 	error `rc'
}
local normal = (${MFpdist}==0)
local nobs = e(nobs)
local rdf = e(tdf)
local bigpwrs `df'
local bigp `bigpwrs'
local scale = e(disp)
local dev2 = e(dev)
/*
	m0 is lowest model entertained.
	m0=-1 => can omit.
*/
if `omit' local m0 -1
else if `m'==0 local m0 0 // highest possible model is linear
else {
	if "`testlinear'"=="notestlinear" local m0 1 // skip testing m=`m' vs. linear
	else local m0 0 // default
}
local hed `vname'
local star " "
local done 0
if `aic'==0 & "`sequential'"=="" {
/*
	Closed tests
*/
	local m1 = `m'+1	// m will typically be 1
	local bigdev `dev`m1''
	if `m'==0 local degmax lin.
	else local degmax gam
	local sig 0
	local j0 `m0'
	while !`done' & `j0'<`m' {
		local j = `j0'+1
		if `j0'==-1 { // null vs. model
			local vs null
			if `m'==0 local n1 1 // null vs. linear
			else local n1 `df'
			local pwrs "."
		}
		else if `j0'==0 { // gam vs linear
			local vs lin.
			local n1 = `df'-1
			local pwrs 1
		}
		local n2 = `rdf'-`n1'
		local d = `dev`j''-`bigdev'
		frac_pv `normal' "`weight'" `nobs' `d' `n1' `n2'
		local P = r(P)
		local pnom = cond(`j0'==-1,`select',`alpha')
		local ss = cond(`j0'==-1,"*","+")
		if `alpha'==1 & `j0'>-1 local j0 `m' // finished
		else if `P'<=`pnom' {
			local sig 1
			local star `ss'
		}
		di in smcl in gr "`hed'" _col(14) "`vs'" /*
		 */ _col(21) "`degmax'" _col(26) %10.3f in ye `dev`j'' /*
		 */ _col(38) %8.3f `d' %7.3f `P' "`star'" /*
		 */ _col(57) "`pwrs'" _col(67) "`bigp'"
		local hed
		local degmax
		local bigp
		if `j0'<`m' & `P'>`pnom' {
			// Non-sig test. j0 = -1 signifies test against null model.
			local done 1
			local dev `dev`j''
			if `j0'==-1 local n // drop
			// OLD, WRONG: else local n = "s_"+substr("`n'", 1, 28)
		}
		local star " "
		if `m'>0 & `j0'==-1 & "`testlinear'"=="notestlinear" {
			local j0 1			/* skip testing linear */
		}
		else local ++j0
	}
	if !`done' {	/* highest model was best */
		local done 1
		local dev `bigdev'
		local pwrs `bigpwrs'
		local n = "s_"+substr("`n'", 1, 28)
	}
}
if `aic' {		/* AIC selection of function and/or variable */
	local ss `star'
	local cz 0
	if `m'==0 {
		local degmax lin.
	}
	else local degmax gam
	if `alpha'==1 {			/* do not optimise function */
		* null model
		GetParam `m0' `cz' `df'
		local vs_0 `s(desc)'
		local df_0 `s(df)'
		local pwrs_0 `s(pwrs)'
		local dev_0 `dev`s(j)''
		* max model
		GetParam `m' `cz' `df'
		local vs_1 `s(desc)'
		local df_1 `s(df)'
		local pwrs_1 `s(pwrs)'
		local dev_1 `dev`s(j)''
		* compute AICs
		forvalues j=0/1 {
			local aic_`j'=`dev_`j''+2*`df_`j''
			di in gr "`hed'" _col(14) "`vs_`j''" /*
			 */ _col(26) %10.3f in ye `dev_`j'' /*
			 */ _col(57) "`pwrs_`j''"
			 local hed
		}
		if `select'==1 | (`select'==-1 & (`aic_1'<`aic_0')) {
			* accept max model
			local sig 1
			local ss "*"
			local n = "s_"+substr("`n'", 1, 28)
			local pwrs `pwrs_1'
			local dev `dev_1'
		}
		else {
			* drop
			local n
			local pwrs `pwrs_0'
			local dev `dev_0'
		}
	}
	else {		/* optimise function */
		local aicopt .
		local jopt .
		forvalues j=`m0'/`m' {
			if "`testlin'"!="notestlinear" | `j'!=0 {
				GetParam `j' `cz' `df'
				local Aic=`dev`s(j)''+2*`s(df)'
				di in gr "`hed'" _col(14) "`s(desc)'" /*
				 */ _col(26) %10.3f in ye `dev`s(j)'' /*
				 */ _col(57) "`s(pwrs)'"
				if `Aic'<`aicopt' {
					local jopt `j'
					local aicopt `Aic'
				}
				local hed
			}
		}
		* Opt values
		GetParam `jopt' `cz' `df'
		local pwrs `s(pwrs)'
		local dev `dev`s(j)''
		if `jopt'==-1 {
			* drop
			local n
		}
		else {
			* accept
			local ss=cond(`m0'==-1,"*","+")
			if `jopt'==1 local n = "s_"+substr("`n'", 1, 28)
		}
	}
}
di in smcl in gr _col(14) "Final" _col(28) in ye %8.3f `dev' _col(57) "`pwrs'"
local rhs "`rhs' `n'"
di
ret local rhs `rhs'
ret local pwrs `pwrs'
ret scalar dev = `dev'
ret local n `n'
end

program define FracAdj, rclass
version 8
* Inputs: 1=macro `adjust', 2=case filter.
* Returns adjustment values in r(adj1),...
* Returns number of unique values in r(uniq1),...

args adjust touse
if "`adjust'"=="" {
	FracDis mean adjust
}
else FracDis "`adjust'" adjust
tempname u
local i 1
while `i'<=$MFP_n {
	local x ${MFP_`i'}
	quietly inspect `x' if `touse'
	scalar `u'=r(N_unique)
	local nx: word count `x'
	if `nx'==1 {	/* can only adjust if single predictor */
		local a ${S_`i'}
		if "`a'"=="" | "`adjust'"=="" {	/* identifies default cases */
			if `u'==1 {		/* no adjustment */
				local a
			}
			else if `u'==2 {	/* adjust to min value */
				quietly summarize `x' if `touse', meanonly
				if r(min)==0 {
					local a
				}
				else local a=r(min)
			}
			else local a mean
		}
		else if "`a'"=="no" {
			local a
		}
		else if "`a'"!="mean" {
			confirm num `a'
		}
	}
	return local adj`i' `a'
	return scalar uniq`i'=`u'
	local i=`i'+1
}
end

program define FracDis
version 8
* Disentangle varlist:string clusters---e.g. for DF.
* Returns values in $S_*.
* If `3' is null, lowest and highest value checking is disabled.

local target "`1'"		/* string to be processed */
local tname "`2'"		/* name of option in calling program */
if "`3'"!="" {
	local low "`3'"		/* lowest permitted value */
	local high "`4'"	/* highest permitted value */
}
tokenize "`target'", parse(",")
local ncl 0 			/* # of comma-delimited clusters */
while "`1'"!="" {
	if "`1'"=="," {
		mac shift
	}
	local ncl=`ncl'+1
	local clust`ncl' "`1'"
	mac shift
}
if "`clust`ncl''"=="" {
	local ncl=`ncl'-1
}
if `ncl'>$MFP_n {
	di as err "too many `tname'() values specified"
	exit 198
}
/*
	Disentangle each varlist:string cluster
*/
local i 1
while `i'<=`ncl' {
	tokenize "`clust`i''", parse("=:")
	if "`2'"!=":" & "`2'"!="=" {
		if `i'>1 {
			noi di as err /*
			*/ "invalid `tname'() value `clust`i''" /*
		 	*/ ", must be first item"
			exit 198
		}
		local 2 ":"
		local 3 `1'
		local j 0
		local 1
		while `j'<$MFP_n {
			local j=`j'+1
			local nxi: word count ${MFP_`j'}
			if `nxi'>1 {
				 local 1 `1' (${MFP_`j'})
			}
			else local 1 `1' ${MFP_`j'}
		}
	}
	local arg3 `3'
	if "`low'"!="" & "`high'"!="" {
		cap confirm num `arg3'
		if _rc {
			di as err "invalid `tname'() value `arg3'"
			exit 198
		}
		if `arg3'<`low' | `arg3'>`high' {
			di as err /*
			*/ "`tname'() value `arg3' out of allowed range"
			exit 198
		}
	}
	while "`1'"!="" {
		gettoken tok 1 : 1
		if substr("`tok'",1,1)=="(" {
			local list
			while substr("`tok'",-1,1)!=")" {
				if "`tok'"=="" {
					di as err "varlist invalid"
					exit 198
				}
				local list "`list' `tok'"
				gettoken tok 1 : 1
			}
			unabbrev `list' `tok'
			local w `s(varlist)'
			FracIn "`w'"
			local v`s(k)' `arg3'
		}
		else {
			unabbrev `tok'
			local tok `s(varlist)'
			local j 1
			local w : word 1 of `tok'
			while "`w'" != "" {
				FracIn `w'
				local v`s(k)' `arg3'
				local j=`j'+1
				local w : word `j' of `tok'
			}
		}
	}
	local i = `i'+1
}
local j 0
while `j'<$MFP_n {
	local j=`j'+1
	if "`v`j''"!="" {
		global S_`j' `v`j''
	}
	else global S_`j'
}
end

program define FracIn, sclass /* target varname/varlist */
version 8
* Returns s(k) = index # of target in MFP varlists.
args v
sret clear
sret local k 0
local j 0
while `j'<$MFP_n {
	local j = `j'+1
	if "`v'"=="${MFP_`j'}" {
		sret local k `j'
		local j $MFP_n
	}
}
if `s(k)'==0 {
   	di as err "`v' is not an xvar"
   	exit 198
}
end

program define FracRep
* 1=descriptor e.g. FRACTIONAL POLYNOMIAL
* 2=param descriptor e.g. df
* 3=param names e.g. powers
version 8
args desc param paramv
local i 1
local l=length("`paramv'")
while `i'<=$MFP_n {
	local l=max(`l',length("`e(Fp_k`i')'"))
	local i=`i'+1
}
local l=`l'+48
local title "Final multivariable `desc' model for `e(fp_depv)'"
local lt=length("`title'")
di in smcl _n in gr "`title'"
di in smcl in gr "{hline 13}{c TT}{hline `l'}"
di in smcl in gr _skip(4) "Variable {c |}" _col(19) "{hline 5}" _col(24) /*
 */ "Initial" /*
 */ _col(31) "{hline 5}" _col(46) "{hline 5}" _col(51) "Final" /*
 */ _col(56) "{hline 5}"
di in smcl in gr _col(14) "{c |} `param'"/*
 */ _col(25) "Select" /*
 */ _col(34) "Alpha" /*
 */ _col(43) "Status" /*
 */ _col(51) "`param'" /*
 */ _col(59) "`paramv'"
di in smcl in gr "{hline 13}{c +}{hline `l'}"
local i 1
while `i'<=$MFP_n {
	local pars `e(Fp_k`i')'
	if "`pars'"=="" | "`pars'"=="." {
		local final 0
		local status out
		local pars
	}
	else {
		local status in
		local final=e(Fp_fd`i')
	}
	local name ${MFP_`i'}
	local skip=12-length("`name'")
	if `skip'<=0 {
		local name=substr("`name'",1,9)+"..."
		local skip 0
	}
	local select=e(Fp_se`i')
	local alpha=e(Fp_al`i')
	if `select'==-1 {
		local select " A.I.C."
	}
	else local select: di %7.4f `select'
	if `alpha'==-1 {
		local alpha " A.I.C."
	}
	else local alpha: di %7.4f `alpha'

	di in smcl in gr _skip(`skip') "`name' {c |}" in ye /*
	 */ _col(19) e(Fp_id`i') /*
	 */ _col(24) "`select'" /*
	 */ _col(33) "`alpha'" /*
	 */ _col(45) "`status'" /*
	 */ _col(53) "`final'" /*
	 */ _col(59) "`pars'"
	local i=`i'+1
}
di in smcl in gr "{hline 13}{c BT}{hline `l'}"
`e(cmd)'
di in smcl in gr "Deviance:" in ye %9.3f e(fp_dev) in gr "."
end

*! version 1.0.0 PR 11Jul2002.
program define TestVars, rclass /* LR-blocktests variables in varlist, adj base */
	version 8
	gettoken cmd 0 : 0, parse(" ")
	local dist $MFpdist
	local normal = (`dist'==0)
	if `normal' 		local family gaussian
	else if `dist'==1 	local family binomial
	else 				local family `cmd'
	if `dist' != 7 {
		local vv varlist(min=2) 
	}
	else {
		local vv varlist(min=1)
	}
	syntax `vv' [if] [in] [aw fw pw iw], /*
	 */ [, DEAD(string) noCONStant BASE(varlist) * ]
	frac_cox "`dead'" `dist'
	if "`constant'"=="noconstant" {
		if `dist'==3 | `dist'==7 {
			di as err "noconstant invalid with `cmd'"
			exit 198
		}
		local options "`options' nocons"
	}
	tokenize `varlist'
	if `dist' != 7 {
		local y `1'
		mac shift
	}
	local rhs `*'
	tempvar touse
	quietly {
		mark `touse' [`weight' `exp'] `if' `in'
		markout `touse' `rhs' `y' `base' `dead'
		if "`dead'"!="" {
			local options "`options' dead(`dead')"
		}
	/*
		Deal with weights.
	*/
		frac_wgt `"`exp'"' `touse' `"`weight'"'
		local wgt `r(wgt)'
		count if `touse'
		local nobs = r(N)
	}
	/*
		Calc deviance=-2(log likelihood) for regression on base covars only,
		allowing for possible weights.
	
		Note that for logit/clogit/logistic with nocons, must regress
		on zero, otherwise get r(102) error.
	*/
	if (`dist'==1 | `dist'==6) & "`constant'"=="noconstant" {
		tempvar z0
		qui gen `z0'=0
	}
* noi di in red `"gam `y' `z0' `base' `wgt' if `touse', df(1) family(`family') `options' `gamopts'"'
	gam `y' `z0' `base' `wgt' if `touse', df(1) family(`family') `options' `gamopts'
	local dev0 = e(dev)
	if `normal' {
		local rsd0=e(rmse)
	}
	/*
		Fit model
	*/
* noi di in red `"gam `y' `rhs' `base' `wgt' if `touse', df(1) family(`family') `options' `gamopts'"'
	gam `y' `rhs' `base' `wgt' if `touse', df(1) family(`family') `options' `gamopts'
	local dev1 = e(dev)
	if `normal' {
		local rsd1=e(rmse)
	}
	local df: word count `rhs'
	local bdf: word count `base'
	local df_r = `nobs'-`df'-`bdf'-("`constant'"!="noconstant")
	local d=`dev0'-`dev1'
	frac_pv `normal' "`wgt'" `nobs' `d' `df' `df_r'
	local P = r(P)
	di in smcl in gr "Deviance 1:" in ye %9.2g `dev1' in gr ". " _cont
	di in smcl in gr "Deviance 0:" in ye %9.2g `dev0' in gr ". "
	di in smcl in gr "Deviance d:" in ye %9.2g `d' in gr ". P = " in ye %8.4f `P'
	* store
	return scalar dev0 = `dev0'
	return scalar dev1 = `dev1'
	if `normal' {
		return scalar s0 = `rsd0'
		return scalar s1 = `rsd1'
	}
	return scalar df_m = `df'
	return scalar df_r = `df_r'
	return scalar devdif = `d'
	return scalar P=`P'
	return scalar N = `nobs'
end

program define GetParam, sclass
/*
	For given model j0, return its descriptor, numerator df and powers.
	cz is catzero indicator.
	j0: -1 null model, 0 linear, >0 gam with `df' edf
	Returns s(j) as index for dev`j' local macros
*/
args j0 cz df
if `j0'==-1 {
	local desc null
	local n1 0
	local pwrs .
}
else if `j0'==0 {
	local desc lin.
	local n1=1+`cz'
	local pwrs 1
}
else {
	local desc gam
	local n1=`df'+`cz'
	local pwrs `df'
}
local j=`j0'+1
sret local j `j'
sret local desc `desc'
sret local pwrs `pwrs'	// really, pwrs is gam df for this variable.
sret local df `n1'
end

program define cmd_chk, sclass
	version 6
	args cmd
	sret clear
	if substr("`cmd'",1,3)=="reg" {
		local cmd regress
	}

	tokenize cox gamma logistic logit poisson regress stcox
	sret local bad 0
	local done 0
	while "`1'"!="" & !`done' {
		if "`1'"=="`cmd'" { local done 1 }
		mac shift
	}
	if !`done' {
		sret local bad 1
		exit
	}
	/*
		dist=0 (normal), 1 (binomial), 2 (poisson), 3 (cox), 4 (glm),
		5 (xtgee), 6(ereg/weibull).
	*/
	if "`cmd'"=="logit" | "`cmd'"=="logistic" {
		sret local dist 1
	}
	else if "`cmd'"=="poisson" {
		sret local dist 2
	}
	else if "`cmd'"=="cox" {
		sret local dist 3
	}
	else if "`cmd'"=="gamma" {
		sret local dist 6
	}
	else if "`cmd'"=="stcox" {
		sret local dist 7
	}
	else sret local dist 0
end
